# coding: utf-8

# # Partner Configurations Helper

from MaaS360APIs import *
import logging

logger = logging.getLogger('PartnerConfig')
handler = logging.FileHandler('partner-integration.log')
formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler) 
logger.setLevel(logging.INFO)

# ## Method to create partner configurations
# ### This Method does the following in given sequence
# 1. Create MaaS360APIsHelper instance for the given customer
# 2. Create boolean and enum type custom attributes
# 3. Create 4 device groups with search criteria based on created custom attributes
# 4. Create Alert for the dashboard where devices with risk posture = Critical will be shown.
# 4. Upload ios app
# 5. Distribute ios app on the created group
# 6. Upload android app
# 7. Distribute android app on the created group

def createPartnerConfigurations(host, billingId, userName, password, appID, appVersion, platformID, appAccessKey):

    # ## Create MaaS360APIsHelper instance for the given customer
    # Instance will be created only if valid credentials are provided
    try:
        logger.debug('Creating MaaS360APIsHelper instance')
        apisHelper = MaaS360APIsHelper(host, billingId, userName, password, appID, appVersion, platformID, appAccessKey)
        logger.info('Done')
    except ValueError as e:
        logger.error('Unable to create MaaS360APIsHelper instance, subsequent api calls will be cancelled, Reason:' + str(e))
        return

    # Creating boolean type custom attribute.
    booleanCustAttributeRequestBody = '''
    {
       "name":"MTD Device",
        "type":"boolean"
    }
    '''
    logger.debug('Calling MaaS360APIsHelper.createCustomAttribute for creating custom attribute "MTD Device"')
    booleanCustAttrPresent = apisHelper.createCustomAttribute(booleanCustAttributeRequestBody)

    if(booleanCustAttrPresent is not None and booleanCustAttrPresent):
        logger.info('Done')
    else:
        logger.error('Custom attribute is not created, subsequent api calls will be cancelled')
        return

    # Creating boolean type custom attribute.
    enumCustAttributeRequestBody = '''
    {
       "name":"zimperium_risk_posture",
        "type":"enum",
        "enumValues": [
               "Normal",
               "Low",
               "Elevated",
               "Critical"
            ],
            "defaultEnumValue": "Normal"
    }
    '''
    logger.debug('Calling MaaS360APIsHelper.createCustomAttribute for creating custom attribute "zimperium_risk_posture"')
    enumCustAttrPresent = apisHelper.createCustomAttribute(enumCustAttributeRequestBody)
    
    if(enumCustAttrPresent is not None and enumCustAttrPresent):
        logger.info('Done')
    else:
        logger.error('Custom attribute is not created, subsequent api calls will be cancelled')
        return

    # ## Create device group with search criteria on created custom attributes
    deviceGroupRequestBody = '''
        {
            "groupName" : "MTD Devices",
            "groupDescription" : "MTD Devices",
            "deviceStatus" : "Active Devices",
            "lastReported" : "All Records",
            "criteriaOperator" : "All Conditions (AND)",
            "conditions" : [
                {
                    "category" : "Custom Attributes",
                    "attribute" : "MTD Device",
                    "criteria" : "Equal To",
                    "value1" : "Yes"
                }
            ]
        }
        '''
    logger.debug('Calling MaaS360APIsHelper.createDeviceGroup for creating device group "MTD Devices"')
    deviceGroupId = apisHelper.createDeviceGroup(deviceGroupRequestBody)

    if(deviceGroupId is True):
        logger.warning('Device Group "MTD Devices" already exist. Upload app will be skipped. Delete the device group and rerun the script.')
    elif (deviceGroupId is False):
        logger.error('Device group "MTD Devices" is not created, subsequent api calls will be cancelled')
        return
    else:
        logger.info('Device Group "MTD Devices" created')

    # ## Create device group MTD Risk Posture Low with search criteria on created custom attribute zimperium_risk_posture = Low
    deviceGroupRequestBodyLow = '''
    {
        "groupName" : "MTD Risk Posture Low",
        "groupDescription" : "MTD Devices with Risk Posture equal to Low.",
        "deviceStatus" : "Active Devices",
        "lastReported" : "All Records",
        "criteriaOperator" : "All Conditions (AND)",
        "conditions" : [
            {
                "category" : "Custom Attributes",
                "attribute" : "zimperium_risk_posture",
                "criteria" : "Equal To",
                "value1" : "Low"
            }
        ]
    }
    '''
    logger.debug('Calling MaaS360APIsHelper.createDeviceGroup for creating device group "MTD Risk Posture Low" group')
    groupIdLow = apisHelper.createDeviceGroup(deviceGroupRequestBodyLow)

    if (groupIdLow is True):
        logger.warning(
            'Device Group "MTD Risk Posture Low" already exist.')
    elif (groupIdLow is False):
        logger.error('Device group "MTD Risk Posture Low" is not created, subsequent api calls will be cancelled')
        return
    else:
        logger.info('Device Group "MTD Risk Posture Low" created')

    # ## Create device group MTD Risk Posture Elevated with search criteria on created custom attribute zimperium_risk_posture = Elevated
    deviceGroupRequestBodyElevated = '''
    {
        "groupName" : "MTD Risk Posture Elevated",
        "groupDescription" : "MTD Devices with Risk Posture equal to Elevated.",
        "deviceStatus" : "Active Devices",
        "lastReported" : "All Records",
        "criteriaOperator" : "All Conditions (AND)",
        "conditions" : [
            {
                "category" : "Custom Attributes",
                "attribute" : "zimperium_risk_posture",
                "criteria" : "Equal To",
                "value1" : "Elevated"
            }
        ]
    }
    '''
    logger.debug('Calling MaaS360APIsHelper.createDeviceGroup for creating device group "MTD Risk Posture Elevated" group')
    groupIdElevated = apisHelper.createDeviceGroup(deviceGroupRequestBodyElevated)

    if (groupIdElevated is True):
        logger.warning(
            'Device Group "MTD Risk Posture Elevated" already exist.')
    elif (groupIdElevated is False):
        logger.error('Device group "MTD Risk Posture Elevated" is not created, subsequent api calls will be cancelled')
        return
    else:
        logger.info('Device Group "MTD Risk Posture Elevated" created')


    # ## Create device group MTD Risk Posture Critical with search criteria on created custom attribute zimperium_risk_posture = Critical
    deviceGroupRequestBodyCritical = '''
    {
        "groupName" : "MTD Risk Posture Critical",
        "groupDescription" : "MTD Devices with Risk Posture equal to Critical.",
        "deviceStatus" : "Active Devices",
        "lastReported" : "All Records",
        "criteriaOperator" : "All Conditions (AND)",
        "conditions" : [
            {
                "category" : "Custom Attributes",
                "attribute" : "zimperium_risk_posture",
                "criteria" : "Equal To",
                "value1" : "Critical"
            }
        ]
    }
    '''
    logger.debug('Calling MaaS360APIsHelper.createDeviceGroup for creating device group "MTD Risk Posture Critical" group')
    groupIdCritical = apisHelper.createDeviceGroup(deviceGroupRequestBodyCritical)

    if (groupIdCritical is True):
        logger.warning(
            'Device Group "MTD Risk Posture Critical" already exist.')
    elif (groupIdCritical is False):
        logger.error('Device group "MTD Risk Posture Critical" is not created, subsequent api calls will be cancelled')
        return
    else:
        logger.debug('Device Group "MTD Risk Posture Critical" created')

    # ## Create a dashboard Alert MTD Risk Posture Critical with search criteria on created custom attribute zimperium_risk_posture = Critical
    alertRequestBody = '''
    {
        "name" : "MTD Risk Posture Critical",
        "description" : "MTD Devices with Risk Posture equal to Critical.",
        "deviceStatus" : "Active Devices",
        "lastReported" : "All Records",
        "availableFor":"All Administrators",
        "criteriaOperator" : "All Conditions (AND)",
        "conditions" : [
            {
                "category" : "Custom Attributes",
                "attribute" : "zimperium_risk_posture",
                "criteria" : "Equal To",
                "value1" : "Critical"                
            }
        ]
    }
    '''
    logger.debug('Calling MaaS360APIsHelper.createAlert for creating dashboard Alert "MTD Risk Posture Critical"')
    alertId = apisHelper.createAlert(alertRequestBody)

    if(alertId is not None):
        logger.info('Done')
    else:
        logger.error('Alert is not created, subsequent api calls will be cancelled')
        #return

    # ## Upload ios app
    # Uploading IBM MaaS360 MTD iTunes app
    iosAppRequestRequestBody = '''
     <appDetails>
        <region>US</region>
        <appId>6479710354</appId>
        <appName>IBM MaaS360 MTD</appName>
        <removeApp>Yes</removeApp>
        <restrictDataBackup>Yes</restrictDataBackup>
        <showInADP>0</showInADP> 
    </appDetails>
    '''
    logger.debug('Calling MaaS360APIsHelper.uploadIosApp to upload ios app "IBM MaaS360 MTD"')
    iosAppId = apisHelper.uploadIosApp(iosAppRequestRequestBody)

    if(iosAppId is not None):
        logger.info('Done')
    else:
        logger.error('Unable to upload ios application')


    # ## Upload android app
    # Uploading IBM MaaS360 MTD play store app
    anroidAppRequestBody = '''
    <appDetails>
        <appSourceURL>https://play.google.com/store/apps/details?id=com.ibm.maas360.android.mtd</appSourceURL>
        <removeAppMDMRemoval>Yes</removeAppMDMRemoval>
        <removeAppSelWipe>Yes</removeAppSelWipe>
        <enforceAuthentication>Yes</enforceAuthentication>
        <enforceCompliance>Yes</enforceCompliance>
        <showInADP>0</showInADP>
    </appDetails>
    '''
    logger.debug('Calling MaaS360APIsHelper.uploadAndroidApp to upload android app "IBM MaaS360 MTD"')
    androidAppId = apisHelper.uploadAndroidApp(anroidAppRequestBody)

    if(androidAppId is not None):
        logger.info('Done')
    else:
        logger.error('Unable to upload android application')
        #return

    # ## Distribute ios app on the created group
    # Distributing ios app on device group created

    # Check to see if ios app uploaded and group is created successfully
    if(iosAppId is not None and deviceGroupId is not True and deviceGroupId is not False):
        iosAppDistributionRequestBody = '''
        {
            "appType":2,
            "appId":"''' + iosAppId + '''",
            "targetDevices":1,
            "deviceGroupId":''' + str(deviceGroupId) + ''',
            "instantInstall":"Yes",
            "retries": 1,
            "sendEmail":"No",
            "sendNotification":"No"
        }
        '''
        logger.debug('Calling MaaS360APIsHelper.distributeApp to distribute ios app with app id: ' + iosAppId)
        iosAddDistributed = apisHelper.distributeApp(iosAppDistributionRequestBody)

        if (iosAddDistributed is not None and iosAddDistributed):
            logger.info('ios application distributed')
        else:
            logger.error('Unable to distribute ios application, subsequent api calls will be cancelled')
            return
    else:
        logger.warning('distribute ios application skipped.')


    # ## Distribute android app on the created group
    # Distributing android app on device group created

    if (androidAppId is not None and deviceGroupId is not True and deviceGroupId is not False):
        androidAppDistributionRequestBody = '''
        {
            "appType":4,
            "appId":"''' + androidAppId + '''",
            "targetDevices":1,
            "deviceGroupId":''' + str(deviceGroupId) + ''',
            "instantInstall":"No",
            "sendEmail":"No",
            "sendNotification":"No"
        }
        '''
        logger.debug('Calling MaaS360APIsHelper.distributeApp to distribute android app with app id: ' + androidAppId)
        andrAppDistributed = apisHelper.distributeApp(androidAppDistributionRequestBody)

        if(andrAppDistributed is not None and andrAppDistributed):
            logger.info('android application distributed')
        else:
            logger.error('Unable to distribute android application, subsequent api calls will be cancelled')
            return
    else:
        logger.warning('distribute android application skipped.')